/*====================================================================================
This source file is part of the TechTools Plugin Development Kit. 
Copyright (c) 2011 by TechTools  

DISCLAIMERS:
 - NO WARRANTIES
     This software is provided 'As Is', without any express or implied
     warranty of any kind, including but not limited to any warranties 
     of merchantability, noninfringement, or fitness of a particular 
     purpose. The Copyright holders do not warrant or assume responsibility
     for the accuracy or completeness of any information contained within
     this software. 
 - LIMITATION OF LIABILITY
     In no event shall the copyright holders be liable for any damages 
     (including, without limitation, lost profits, business interruption, 
     or lost information) rising out of use of or inability to use this 
     software, even if advised of the possibility of such damages.  In no
     event will the copyright holders be liable for loss of data or for 
     indirect, special, incidental, consequential (including lost profit),
     or other damages based in contract, tort or otherwise. The copyright
     holders shall have no liability with respect to the content of the 
     software or any part thereof, including but not limited to errors or 
     omissions contained therein, libel, infringements of rights of publicity,
     privacy, trademark rights, business interruption, personal injury, 
     loss of privacy, moral rights or the disclosure of confidential
     information.  

Use and redistribution of this software or of derived works, in source or
compiled form is permitted as long as the following restrictions are observed:

- The above copyright(s), disclaimers and these restrictions are retained in the
  source files and, if distributed in compiled form, must be duplicated in 
  documentation or other materials and provided with the distribution.
- The derived work is used exclusively as a plugin to the TechTools DigiView 
  software, to process data captured with TechTools hardware.
- The copyright holders' names may not be used to endorse or to promote any 
  product or derived works.

Any other use is prohibited without express, written permission from TechTools.

email: support@tech-tools.com,  sales@tech-tools.com 
web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/


#include "../plugin.h"

// mini plugin based on the ASYNC preparser.  It simply prints each character to a field
// while monitoring the time between characters. If the time between characters exceeds 
// the timeout value, it halts the auto run, forces a save of the capture or vetoes 
// a save, depending on user preferences.
//

// ASYNC specific event flags
#define BRKEVENT   0x80
#define ENDEVENT   0x40
#define PERROR     0x20
#define FERROR     0x10
#define ADDRBYTE   0x02
#define DATAEVENT  0x01

static int64 lasttimestamp;
static Data64 TimeScale;
static Data64 timeout;
static Data64 starttime;   // first raw sample time
static Data64 finaltime;   // final raw sample time
static bool haltontimeout;
static bool saveontimeout;
static bool nosaveontimeout;
static bool showTOfield;
static bool timedout;
static bool dotimeout;

void OnLoad() {}       // no one-time initialization needed
void OnUnLoad() {}     // no final cleanup needed

enum cfg_items {CIhalt,CIsave,CInosave,CIshowtofield,CItimeout,CIUNKNOWN};
enum field_formats {FFdata,FFperr,FFferr,FFbrk,FFtimeout,FFUNKNOWN};

void GetStrList(int ID, vector<string> &strl)
{	
	switch (ID) 
	{
	case 0: strl.push_back("AsyncWD example V1.0"); break;
	case 1: 
		strl.assign(CIUNKNOWN,"");
		strl[CIhalt]    = "Halt on timeout,checkbox,false";
		strl[CIsave]    = "Force Save on timeout,checkbox,false";
		strl[CInosave]  = "Veto save on timeout,checkbox,false";
		strl[CIshowtofield]  = "Show TIMEOUT fields,checkbox,true";
		strl[CItimeout] = "Timeout (from Start bit to Start bit),timeedit,0";
		break;
	case 2: // field formats 
		strl.assign(FFUNKNOWN,"");
		strl[FFdata] = "DATA,,,{7:0}";
		strl[FFperr] = "ParityError,RED,WHITE,P";
		strl[FFferr] = "FrameError,RED,WHITE,F";
		strl[FFbrk]  = "BREAK,YELLOW,BLACK,BREAK";
		strl[FFtimeout] ="Timeout,ORANGE,BLACK,Timeout";
		break; 
	case 3:  strl.push_back("ASYNC"); break;      // use ASYNC pre-parser
	case 4:  strl.push_back("1");	break;        // require framework version 1 or above
		// 5 (lookuptables) returns empty list
	}
}

 // set global configuration items
void SetInitItem(unsigned char ID, unsigned char subID, int value) 
{
	switch (ID)
	{
	case 0:  // TimeScale
		switch (subID)
		{
		case 0: TimeScale.lowint = value; break;
		case 1: TimeScale.highint = value; break;
		}
		break;
	case 1:  // start time
		switch (subID)
		{
		case 0: starttime.lowint = value; break;
		case 1: starttime.highint = value; break;
		}
		break;
	case 2:  // final time
		switch (subID)
		{
		case 0: finaltime.lowint = value; break;
		case 1: finaltime.highint = value; break;
		}
		break;
	}
}

    // set user configuration items
	void SetCfgItem(unsigned char ID, unsigned char subID, int value) 
	{ 
		switch (ID)
		{
		case CIhalt: if (subID == 0) haltontimeout = (value != 0); break;
		case CInosave: if (subID == 0) nosaveontimeout = (value != 0); break; 
		case CIsave: if (subID == 0) saveontimeout = (value != 0); break; 
		case CIshowtofield: if (subID == 0) showTOfield = (value != 0); break; 
		case CItimeout:
			if (subID == 0)      timeout.lowint = value;
			else if (subID == 1) timeout.highint = value; 
			break;
		}
	}


	void StartOfData() 
	{   
		timedout = false;
		timeout.i64 *= TimeScale.i64;  // scale the timeout
		lasttimestamp = starttime.i64;  // use first raw data sample time as the reference  
		dotimeout = (timeout.i64 != 0);
	}


	void Parse(int64 timestamp, Data64 data)
	{
		if (data.bytes[6] & DATAEVENT)
		{   
			if (dotimeout && (!timedout || showTOfield ) && ((timestamp - lasttimestamp) > timeout.i64)) 
			{	
				timedout = true;
				if (showTOfield) StartField(lasttimestamp+timeout.i64,data,FFtimeout);
			}

			StartField(timestamp,data,FFdata);  // output this symbol with DATA field format
		}  // data event

		// note use of 'else'. Multiple event flags can be set along with the data event.  However, we
		// will receive additional events later to handle the errors.  The flags are sert in the data
		// as well so we can have early notice.  We choose to ignore them at this time.

		else if (data.bytes[6] & PERROR)  // handle parity error events
			StartField(timestamp,data,FFperr);  // output it with Parity error field format

		else if (data.bytes[6] & FERROR)  // handle framing error events
			StartField(timestamp,data,FFferr);  // output it with Frame error field format

		else if (data.bytes[6] & BRKEVENT)  // handle BREAK events
			StartField(timestamp,data,FFbrk);  // output it with BREAK field format 

		else if (data.bytes[6] & ENDEVENT)  // handle end-of-char events
  		    EndField(timestamp);

		lasttimestamp = timestamp;   // reset watchdog on any event

	}


	void EndOfData() 
	{
		// check the time between the final character and the final raw data sample
		if (!timedout && ((finaltime.i64 - lasttimestamp) > timeout.i64))  // timeout...
			timedout = true;

		if (timedout)
		{     // note timestamp, data and formatID are ignored on these calls
			if (haltontimeout)
				SendControl(finaltime.i64,CNTLHALT);  // send halt contorl
			if (saveontimeout)
				SendControl(finaltime.i64,CNTLSAVE);  // send save control
			if (nosaveontimeout)
				SendControl(finaltime.i64,CNTLNOSAVE);  // send no-save control
		}
	}
