/*====================================================================================
This source file is part of the TechTools Plugin Development Kit. 
Copyright (c) 2011 by TechTools  

DISCLAIMERS:
- NO WARRANTIES
This software is provided 'As Is', without any express or implied
warranty of any kind, including but not limited to any warranties 
of merchantability, noninfringement, or fitness of a particular 
purpose. The Copyright holders do not warrant or assume responsibility
for the accuracy or completeness of any information contained within
this software. 
- LIMITATION OF LIABILITY
In no event shall the copyright holders be liable for any damages 
(including, without limitation, lost profits, business interruption, 
or lost information) rising out of use of or inability to use this 
software, even if advised of the possibility of such damages.  In no
event will the copyright holders be liable for loss of data or for 
indirect, special, incidental, consequential (including lost profit),
or other damages based in contract, tort or otherwise. The copyright
holders shall have no liability with respect to the content of the 
software or any part thereof, including but not limited to errors or 
omissions contained therein, libel, infringements of rights of publicity,
privacy, trademark rights, business interruption, personal injury, 
loss of privacy, moral rights or the disclosure of confidential
information.  

Use and redistribution of this software or of derived works, in source or
compiled form is permitted as long as the following restrictions are observed:

- The above copyright(s), disclaimers and these restrictions are retained in the
source files and, if distributed in compiled form, must be duplicated in 
documentation or other materials and provided with the distribution.
- The derived work is used exclusively as a plugin to the TechTools DigiView 
software, to process data captured with TechTools hardware.
- The copyright holders' names may not be used to endorse or to promote any 
product or derived works.

Any other use is prohibited without express, written permission from TechTools.

email: support@tech-tools.com,  sales@tech-tools.com 
web: www.tech-tools.com   Voice:972-272-9392  FAX: 972-494-5814
======================================================================================*/

// GroupFilter.cpp
// This is a simple example of a full plugin. (Uses the built-in RAW parser as a
// preparser.)  Its purpose is to demonstrate a basic Group Filter. It filters across
// all channels involved in a signal, eliminating any glitches before the data is interpreted.
// This example simply displays the selected channels as a BUS (after filtering them).

#include "../plugin.h"

// local prototypes
bool GroupFilter(int64 &timestamp, Data64 &rawdata);
void GroupFilterInit(int64 timestamp, Data64 rawdata,Data64 mask, int width);


// A little short-hand... These give readable names to the Configuration Options and
// field formats described below.  The final entries are used as sentinals to mark the length/ends  

//  configuration items (case 1)
static enum cfgnames { MYDATA_MASK,GLITCH,CFGEND};

// unused stubs
void OnLoad() {}
void OnUnLoad() {}
void EndOfData() {} // we do not need to do any clean up.


// called to retrieve descriptions and configuration information from the plug-in
// fills the provided vector container with the requested set of strings
void GetStrList(int ID, vector<string> &strl)
{
	// You are free to store your strings any way you wish as long as you
	// copy them into the provided vector when requested. You might choose
	// to store them as constant arrays or pre-initialized vectors.  We chose
	//to load them directly in-line here, keeping all configuration in one place.
	switch (ID)
	{ 
	case 0:   // Plug-in Description
		// These lines are displayed in the signal editor for identification
		// but serve no other purpose. For best results, keep to 4 or fewer lines.
		strl.push_back(	"Example Group Filter plugin V1.0");
		break;

	case 1:   // configuration options
		// These are the items displayed in the SIGNAL EDITOR. They allow the
		// user to configure the plug-in for a given signal. You must provide
		// at least 1 chanselect type option to allow the user to assign physical
		// channel(s) to the signal. 
		strl.assign(CFGEND,"");  // allocate and init the list
		strl[MYDATA_MASK]      =  "Data Channels,chanselect,2,1,36,false,false";
		strl[GLITCH]           =  "Filter Glitches <= (ns),slider,0,0,128,4";
		break;

	case 2:  // Field Formats
		// These tell the app how to display your fields. You must define at least 
		// one field format.
		strl.push_back(	"Data,c0c0c0,BLACK,{}");
		break;

	case 3:  //return which preparser we need
		strl.push_back("RAW");
		break;

	case 4: // return minimim framework ver we need
		strl.push_back("1");
		break;
	}
}


static bool first_sample;
static uint64 BusHighBit;
static uint64 BusLowBit;

// our configuration items
static Data64 datamask;
static int TimeScale;
static int GlitchWidth;


// called to set global configuration items
// we only care about the timescale (always an int so ignore highint)
void SetInitItem(unsigned char ID, unsigned char subID, int value) 
{
	if ((ID == 0) && (subID == 0))  // lower int of TimeScale
		TimeScale = value;
	
	// ignore any other init items sent to us.
}



// called to set our configuration options.
// These represent the choices the user made in response to your Configuration Options
// You can store and retrieve your configuration items any way you want.
// We chose to store them directly in named variables.
// The ID corresponds to the index of the Configuration Options item.
// eg 0 => the first configuration string you provided....

void SetCfgItem(unsigned char ID, unsigned char subID, int value)
{
	switch (ID)
	{
	case MYDATA_MASK:  // data bus mask
		switch (subID)
		{
		case 1: datamask.lowint = value;   break;
		case 2: datamask.highint = value;  break;
		}
		break;

	case GLITCH:
		if (subID == 0) GlitchWidth = value;
		break;

		// ignore illegal cfg items
	}
}



// Called immediately after setting configuration options and before data
// streaming begins. Do any initialization you require here. In particular,
// note that you should assume that the configuration options have changed.
void StartOfData()
{	// some of our initialization is dependent on SEEing the first sample...
	// do it in the first call to 'parse'
	first_sample = true;

	// Pre-calculate the bus-type signal's upper and lower channel usage.  This greatly
	// speeds up the data extraction process during data streaming ('parse' calls).
	FindChannelLimits(datamask.i64,BusHighBit,BusLowBit);
}


// Called when a new data sample has been received.
// This is the protocol parser itself.  It must examine each data sample as it
// arrives, send back field information when appropriate for this protocol and
// return immediately.  It must never loop or attempt to use stdio.  It must 
// maintain its state between calls through statics.
// NOTE that this could get called on the order of 250,000 times per capture per signal,
// so efficiency is important.  We pre-converted many of the configurtion items to 
// bools in the setcfg routine to minimize masking, shifting and int64 compares here.
void Parse(int64 timestamp, Data64 rawdata)
{
	Data64 mydata;

	// if this is the first sample, set the filter parameters and
	// prime it with the first sample, then RETURN
	if (first_sample)
	{ 
		first_sample = false;  // only one 'first sample'

		// inititalize the filter with the first sample, the filter width and a channel mask
		// The channel mask is normally an OR of all of the channel select masks. In this 
		// example, there is only 1 channel select object so we just use it.
		// Note that we scale the glitch width value by the TimeScale (see the developers guide)
		GroupFilterInit(timestamp,rawdata,datamask,GlitchWidth * TimeScale);

		return;
	}


	// send current data to filter.  If the filter has data for us, it updates the time/data params
	// and returns TRUE.
	if (GroupFilter(timestamp,rawdata))
	{ // filter returned some data (in timestamp&rawdata)...process it.

		// extract and pack the bus value
		mydata.i64 = pack(rawdata.i64, datamask.i64,BusHighBit,BusLowBit);

		// in this example, we simply return the filtered data in a field using the first field format specifier.
		StartField(timestamp, mydata, 0); 
	}
}






/*====================================================================================
Filter Routines
 This implements a GROUP filter. It will filter pulses shorter than the specified 
 time.  It does NOT filter each channel indvidually.
 As a result, it will filter out any transitions that occur near each other, even if 
 on different channels.  In other words, it will not return a valid sample until
 ALL channels have been stable for the specified filter time.  This is helful when 
 looking at bus type signals in that it eliminates bus settling noise. This approach
 is also compatible (if not helpful) with most serial protocols and state type signals
 since they typically only care about the active edge of the clock and they expect the
 remaining signals to be stable at that point.
=====================================================================================*/

static int64 ValidT;   // last validated sample returned
static Data64 ValidD;
static int64 EvalT;    // sample we are currently evaluating
static Data64 EvalD;
static bool HaveValid;  // flag to indicate we have returned at least 1 valid sample
static int FilterWidth; // the minimum pulse width allowed (in scaled ns)
static Data64 FilterMask; // a mask indicating ALL channels used by this signal


void GroupFilterInit(int64 timestamp, Data64 rawdata,Data64 mask, int width)
{
	FilterWidth = width;
	FilterMask = mask;
	EvalT = timestamp;   // the sample we will evaluate next
	EvalD = rawdata;
	HaveValid = false;   // have not returned a valid sample yet
}

bool GroupFilter(int64 &timestamp, Data64 &rawdata)
{

// ignore duplicate data
if ((rawdata.i64 & FilterMask.i64) == (EvalD.i64 & FilterMask.i64)) return 0;

if (((timestamp - EvalT) <= FilterWidth) ||   // has not been stable long enough
	(HaveValid && (ValidD.i64 == EvalD.i64))) // or same as last valid sample returned
{
	// discard eval sample, make the new sample the next one to evaluate
	EvalT = timestamp;
	EvalD = rawdata;
	return 0;
}

// else the eval sample is good, return it to caller and prepare to
// evaluate the next sample
ValidT= EvalT;       // eval sample was valid, remember it
ValidD = EvalD;
EvalT = timestamp;   // current sample is the next one to be evaluated
EvalD = rawdata;
timestamp = ValidT;  // return the validated sample
rawdata = ValidD;
HaveValid = true;    // we have returned at least 1 valid sample
return 1;

}

